import {Component, ViewChild, OnInit} from "@angular/core";
import 'rxjs/add/operator/switchMap';

import * as wjGrid from 'wijmo/wijmo.angular2.grid';
import * as wjcCore from 'wijmo/wijmo';
import * as wjcGrid from 'wijmo/wijmo.grid';
import * as wjcFilter from 'wijmo/wijmo.grid.filter';
import * as wjInput from 'wijmo/wijmo.angular2.input';

import {
  Order,
  Product
} from '../../shared/sdk/models';
import {
  ProductApi
} from '../../shared/sdk/services';
import {
  DialogService
} from '../../services';

@Component({
  selector: 'ts',
  templateUrl: './html.component.html'
})
export class OrderFormComponent implements OnInit {
  @ViewChild('flex') flex: wjGrid.WjFlexGrid;
  @ViewChild('flexFilter') flexFilter: wjcFilter.FlexGridFilter;
  public data: wjcCore.CollectionView;
  public allProducts: Product[];
  public timeOut: any;
  constructor(private productApi: ProductApi) {
  }

  formatter = (panel, r, c, cell) => {
    if (panel.cellType == wjcGrid.CellType.Cell) {
      let flex = panel.grid;
      //if its calculated column

      let column = flex.columns[c];
      if (column.name == 'extPrice') {
        let dataItem = flex.rows[r].dataItem;

        if (dataItem != undefined && dataItem != null) {
          // this.itemAutoCalculate(dataItem);
        }
      }
    }
  };

  productAutoCompleteFormatter = (index, content) => {
    let product = this.allProducts[index];
    content = product.productId + " - " + product.name;
    return content;
  };

  ngOnInit() {
    this.flex.rows.defaultSize = 31;
  };

  onRowAdded = (e) => {
    let flex = e.panel.grid;
    let dataItem = flex.rows[e.row].dataItem;
    this.initialNewItem(dataItem);
  };

  autoCompleteInitialized = (s: wjInput.WjAutoComplete, e) => {
    s.focus();
    console.log("Current text is " + s.text);
    let template = s.getTemplate();
    console.log("template " + template);
    this.onLostFocus(s, e);
  };

  selectProduct(product) {
    // this.flex.startEditing();
    let item = this.flex.selectedItems[0];

    for (let p in product) {
      if (!(typeof product[p] === 'function')) {
        item.product[p] = product[p];
      }
    }
    item.productId = product.productId;
    item.product_id = product.id;
  }

  initialNewItem(dataItem) {
    let orderDetail = new OrderDetail();
    orderDetail.uom = '';
    orderDetail.ordQty = 0;
    orderDetail.invQty = 0;
    orderDetail.price = 0.00;
    orderDetail.discount = 0.00;
    orderDetail.netWeight = 0.00;
    orderDetail.ordWeight = 0.00;
    for (let obj in orderDetail) {
      if (!(typeof orderDetail[obj] === 'function')) {
        dataItem[obj] = orderDetail[obj];
      }
    }
    let product = new Product();
    product.productId = null;
    dataItem.product = product;
  }

  getFocus = (s: wjInput.WjAutoComplete, event) => {
    console.log("getFocus s.text: " + s.text + "," + new Date());
    console.log("getFocus s.selectedValue: " + s.selectedValue + "," + new Date());
  };
  onTextChanged = (s: wjInput.WjAutoComplete, event) => {
    console.log("onTextChanged s.text: " + s.text + "," + new Date());
    console.log("onTextChanged s.selectedValue: " + s.selectedValue + "," + new Date());
  };

  onLostFocus = (s: wjInput.WjAutoComplete, event) => {
    console.log("onLostFocus s.text: " + s.text + "," + new Date());

    let query = s.text;
    if (ValidationService.isNotEmpty(query)) {
      query = CompleteUtilsService.productIdComplete(query);
      let item = this.flex.selectedItems[0];
      if (this.timeOut) {
        clearTimeout(this.timeOut);
      }
      this.timeOut = setTimeout(() => {
          item.product.productId = query;
          this.initialItemByProductAll(item, query);
      }, 100);
    }
  };
  //first set item uom and then set item price
  initialItemByProductAll(item, query) {
    // console.log("initialItemByProductAll,item.product.productId:" + item.product.productId);
    this.productApi.getSpecificProductByCustomerInfo(query).subscribe(
      (result) => {
        if (result.success) {
          item.product = result.data.singleData;
        } else {
          this.failMethod(result, item, query);
        }
      },
      err => {
        return;
      }
    );
  }

  failMethod(result, item, query) {
    this.data.editItem(item);
    this.initialNewItem(item);
    if (query) {
      item.product.productId = query;
    }
    this.data.commitEdit();
    console.log("failMethod:item.product.productId:" + item.product.productId);
    let row = this.data.items.indexOf(item);
    this.flex.startEditing(true, row, 1, true);

  }

  filterProduct = (query, max, callback) => {
    if (query != undefined && query != null && query.trim() != '') {
      this.allProducts = [];
      this.productApi.findProductForAutoComplete(query, max).subscribe(
        (products: Product[]) => {
          this.allProducts = products;
          callback(products);
        },
        err => {

        }
      );
    } else {
      let item = this.flex.selectedItems[0];
      if (item.product != undefined && item.product != null
        && item.product.productId != undefined && item.product.productId != null && item.product.productId.trim() != '') {
        this.allProducts = new Array();
        this.allProducts.push(item.product);
      } else {
        this.allProducts = new Array();
      }
      callback(this.allProducts);
    }
  };
}
